/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ModProxyBalancerTest.java 2866 2008-12-04 15:46:32Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.action.modProxyBalancer;

import java.io.BufferedReader;
import java.io.FileReader;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 * Tests the mod_proxy_balancer manager.
 * 
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
public class ModProxyBalancerTest {

    private ModProxyBalancer modProxyBalancer;

    /**
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception {
        this.modProxyBalancer = new ModProxyBalancer("jkManagerUrl");
    }

    /**
     * @throws java.lang.Exception
     */
    @After
    public void tearDown() throws Exception {
        this.modProxyBalancer = null;
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.action.modProxyBalancer.ModProxyBalancer#ParseAndUpdateModProxyBalancer(String)}
     * .
     */
    @Test
    public void testParseAndUpdateModProxyBalancer() throws Exception {
        this.modProxyBalancer.parseAndUpdateModProxyBalancer(this
            .readFileAsString("src/test/resources/mod_proxy_balancer_htmlResponse.html"));
        Assert.assertEquals(this.modProxyBalancer.balancers.size(), 1);
        Assert.assertEquals(this.modProxyBalancer.nonce, "0586df61-8403-d449-b605-01cb89773ca2");
        Assert.assertEquals(this.modProxyBalancer.balancers.get(0).getName(), "loadbalancer");
        Assert.assertEquals(this.modProxyBalancer.balancers.get(0).getWorkers().size(), 2);
        Assert.assertEquals(this.modProxyBalancer.balancers.get(0).getWorkers().get(0).getUrl(), "http://localhost:9000/");
        Assert.assertEquals(this.modProxyBalancer.balancers.get(0).getWorkers().get(0).isEnabled(), true);
        Assert.assertEquals(this.modProxyBalancer.balancers.get(0).getWorkers().get(1).getUrl(), "http://127.0.0.1:9000/");
        Assert.assertEquals(this.modProxyBalancer.balancers.get(0).getWorkers().get(1).isEnabled(), true);
    }

    private String readFileAsString(final String filePath) throws java.io.IOException {
        StringBuffer fileData = new StringBuffer(1000);
        BufferedReader reader = new BufferedReader(new FileReader(filePath));
        char[] buf = new char[1024];
        int numRead = 0;
        while ((numRead = reader.read(buf)) != -1) {
            String readData = String.valueOf(buf, 0, numRead);
            fileData.append(readData);
            buf = new char[1024];
        }
        reader.close();
        return fileData.toString();
    }

}
