/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GetServerProgressListCommand.as 2883 2008-12-11 13:35:39Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.jadort.client.flex.command {

    import com.adobe.cairngorm.commands.*;
    import com.adobe.cairngorm.control.*;
    import flash.utils.clearTimeout;
    import mx.collections.ArrayCollection;
    import mx.rpc.IResponder;
    import mx.rpc.events.FaultEvent;
    import mx.rpc.events.ResultEvent;
    import org.ow2.jasmine.jadort.client.flex.business.*;
    import org.ow2.jasmine.jadort.client.flex.event.*;
    import org.ow2.jasmine.jadort.client.flex.model.*;
    import org.ow2.jasmine.jadort.client.flex.vo.*;

    public class GetServerProgressListCommand extends SequenceCommandWithErrorHandling implements IResponder {

        private static var my_timedProcess : uint = 0;

        override public function execute(event : CairngormEvent) : void {

            // delegate to the GetServerProgressListDelegate to call the getDeployment service
            var delegate : GetServerProgressListDelegate = JadortModelLocator.getInstance().getServerProgressListDelegete();
            delegate.responder = this;
            delegate.getServerProgressList();
        }

        /**
         * Manage the result found by a delegate.
         */
        public function result(event : Object) : void {

            var resultEvent : ResultEvent = event as ResultEvent;
            var arrayCollection : ArrayCollection = resultEvent.result as ArrayCollection;

            var model : JadortModelLocator = JadortModelLocator.getInstance();
            model.serverProgressList = arrayCollection;

            // Update UI
            CairngormEventDispatcher.getInstance().dispatchEvent(new ServerWorkerProgressEvent());

            // Refresh in 3 seconds
            if (my_timedProcess == 0) {
                my_timedProcess = flash.utils.setTimeout(refreshServerProgress, 3000);
            }
        }

        public function refreshServerProgress() : void {
            flash.utils.clearTimeout(my_timedProcess);
            my_timedProcess = 0;

            var serverRunning : Boolean = false;
            var model : JadortModelLocator = JadortModelLocator.getInstance();
            if (model.serverProgressList != null) {
                for each(var serverProgress : ServerProgressVO in model.serverProgressList) {
                    if (serverProgress.state.toLowerCase().indexOf("running") != -1) {
                        serverRunning = true;
                        break;
                    }
                }
            }

            // if:
            //     - at least one server has not finished
            //     - or we're in the "undeploy old version" step
            //     - or we're in a "maintenance" operation
            // we need to recall the dispatch GetServerProgressListEvent to update the UI
            // (server progress and/or number of active sessions)
            if (serverRunning || model.index == 5 || model.index == 8 || model.index == 9) {
                nextEvent = new GetServerProgressListEvent();
                executeNextCommand();
            }
        }
    }

}
