/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GetServersCommand.as 2883 2008-12-11 13:35:39Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.client.flex.command {

    import com.adobe.cairngorm.commands.*;
    import com.adobe.cairngorm.control.*;
    import flash.utils.clearTimeout;
    import mx.collections.ArrayCollection;
    import mx.rpc.IResponder;
    import mx.rpc.events.FaultEvent;
    import mx.rpc.events.ResultEvent;
    import org.ow2.jasmine.jadort.client.flex.business.*;
    import org.ow2.jasmine.jadort.client.flex.event.*;
    import org.ow2.jasmine.jadort.client.flex.model.*;
    import org.ow2.jasmine.jadort.client.flex.vo.*;

    /**
     * This class allows to get the list of servers
     */
    public class GetServersCommand extends SequenceCommandWithErrorHandling implements IResponder {

        private static var my_timedProcess : uint = 0;

        override public function execute(event : CairngormEvent) : void {

            // delegate to the GetServersDelegate to call the getServers service
            var delegate : GetServersDelegate = JadortModelLocator.getInstance().getServersDelegate();
            delegate.responder = this;
            delegate.getServers();
        }

        /**
         * Manage the result found by a delegate.
         */
        public function result(event : Object) : void {

            var resultEvent : ResultEvent = event as ResultEvent;
            var arrayCollection : ArrayCollection = resultEvent.result as ArrayCollection;

            var model : JadortModelLocator = JadortModelLocator.getInstance();

            // Set the maintained and selected servers
            // Create a dummy list to ensure ArrayCollection.contains works
            var selectedServers : ArrayCollection = new ArrayCollection();
            for each(var server : ServerVO in arrayCollection) {
                if (model.servers != null) {
                    for each(var oldServer : ServerVO in model.servers) {
                        if (oldServer.id == server.id) {
                            if (model.selectedServers.contains(oldServer)) {
                                selectedServers.addItem(server);
                            }
                            break;
                        }
                    }
                }
            }

            // Update the model
            model.servers = arrayCollection;
            model.selectedServers = selectedServers;
            model.nextButtonEnabled = selectedServers.length > 0;

            // Refresh in 3 seconds
            if (my_timedProcess == 0) {
                my_timedProcess = flash.utils.setTimeout(refreshGetServers, 3000);
            }
        }

        public function refreshGetServers() : void {
            flash.utils.clearTimeout(my_timedProcess);
            my_timedProcess = 0;

            var model : JadortModelLocator = JadortModelLocator.getInstance();
            if (model.index == 7) {
                nextEvent = new GetServersEvent();
                executeNextCommand();
            }
        }
    }
}
