/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SelectOperationCommand.as 2885 2008-12-11 14:12:43Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.client.flex.command {

    import com.adobe.cairngorm.commands.*;
    import com.adobe.cairngorm.control.*;
    import mx.rpc.IResponder;
    import mx.rpc.events.FaultEvent;
    import mx.rpc.events.ResultEvent;
    import org.ow2.jasmine.jadort.client.flex.business.*;
    import org.ow2.jasmine.jadort.client.flex.event.*;
    import org.ow2.jasmine.jadort.client.flex.model.*;
    import org.ow2.jasmine.jadort.client.flex.vo.*;


    /**
     * This class allows to select an Operation from the operation list
     * and load the needed objects to resume this operation
     */
    public class SelectOperationCommand extends SequenceCommandWithErrorHandling implements IResponder {

        override public function execute(event : CairngormEvent) : void {

            // cast the cairngorm event so we can get at the selected operation by the UI.
            var selectOperationEvent : SelectOperationEvent = event as SelectOperationEvent;
            // delegate to the SelectOperationDelegate to call the selectOperation() service
            var delegate : SelectOperationDelegate = JadortModelLocator.getInstance().getSelectOperationDelegate();
            delegate.responder = this;
            delegate.selectOperation(selectOperationEvent.eOperation);
        }

        /**
         * Manage the result found by a delegate.
         */
        public function result(event : Object) : void {
            var resultEvent : ResultEvent = event as ResultEvent;
            var selectedOperation : OperationStateVO = resultEvent.result as OperationStateVO;
            // Gets the Model
            var model : JadortModelLocator = JadortModelLocator.getInstance();

            // we load the needed objects to resume the selected operation
            model.selectedOperation = selectedOperation;
            if (selectedOperation.topology != null) {
                model.groups = selectedOperation.topology.groups;
            }
            if (selectedOperation.selectedGroup != null) {
                model.selectedGroup = selectedOperation.selectedGroup;
                model.servers = selectedOperation.selectedGroup.servers;
            }
            if (selectedOperation.application != null) {
                model.uploadedApplication = selectedOperation.application;
                model.selectedGroup = selectedOperation.selectedGroup;
                model.servers = selectedOperation.selectedGroup.servers;
            }
            if (selectedOperation.serverProgressList != null) {
                model.serverProgressList = selectedOperation.serverProgressList;
            }
            
            if (selectedOperation.workerProgressList != null) {
                model.workerProgressList = selectedOperation.workerProgressList;
            }

            // go the step where the operation has stopped
            var getPageIndexEvent : GetPageIndexEvent = new GetPageIndexEvent();
            CairngormEventDispatcher.getInstance().dispatchEvent(getPageIndexEvent);
        }
    }

}