/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerWorkerProgressEvent.as 2883 2008-12-11 13:35:39Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.client.flex.event {

    import com.adobe.cairngorm.control.*;
    import org.ow2.jasmine.jadort.client.flex.model.*;
    import org.ow2.jasmine.jadort.client.flex.vo.*;

    /**
     * Updates the global server and worker progress bar.
     */
    public class ServerWorkerProgressEvent extends JadortEvent {

        /**
         * Name of the Event.
         */
        public static const EVENT_PROGRESS : String = "progress";

        private var progressPercent : Number;
        private var progressError : Boolean;

        public function ServerWorkerProgressEvent() {
            super(EVENT_PROGRESS);

            var canGoToNextStepEvent : CanGoToNextStepEvent = new CanGoToNextStepEvent();
            CairngormEventDispatcher.getInstance().dispatchEvent(canGoToNextStepEvent);

            var canGoToPreviousStepEvent : CanGoToPreviousStepEvent = new CanGoToPreviousStepEvent();
            CairngormEventDispatcher.getInstance().dispatchEvent(canGoToPreviousStepEvent);

            progressError = false;
            progressPercent = 0;

            var DeployCount : Number = 0;
            var model : JadortModelLocator = JadortModelLocator.getInstance();

            if (model.serverProgressList != null) {
                for each(var serverProgress : ServerProgressVO in model.serverProgressList) {
                    if (serverProgress.state.toLowerCase().indexOf("error") != -1) {
                        progressError = true;
                    }
                    progressPercent += serverProgress.progress;
                    DeployCount++;
                }
            }

            if (model.workerProgressList != null) {
                for each(var workerProgress : WorkerProgressVO in model.workerProgressList) {
                    if (workerProgress.state.toLowerCase().indexOf("error") != -1) {
                        progressError = true;
                    }
                    progressPercent += workerProgress.progress;
                    DeployCount++;
                }
            }

            if (DeployCount > 0) {
                progressPercent /= DeployCount;
            }
        }

        public function get ProgressPercent() : Number {
            return this.progressPercent;
        }

        public function get ProgressError() : Boolean {
            return this.progressError;
        }
    }
}
