/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JadortModelLocator.as 2913 2008-12-18 12:08:01Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.client.flex.model {

    import com.adobe.cairngorm.model.IModelLocator;
    import com.adobe.cairngorm.control.CairngormEventDispatcher;
    import mx.collections.ArrayCollection;
    import org.ow2.jasmine.jadort.client.flex.business.*;
    import org.ow2.jasmine.jadort.client.flex.vo.*;
    import org.ow2.jasmine.jadort.client.flex.event.*;

    /**
     * this class is the model locator of the application, we use it to store the client side model.
     *(it is implement the model in the Cairngorm Model-View-Controller architecture)
     */
    [Bindable]
    public class JadortModelLocator implements IModelLocator {

        private static var modelLocator : JadortModelLocator = null;

        // The ServerProgressVO and WorkerProgressVO ActionScript classes add
        // some attributes to their Java correspondents, and the Flash
        // on-the-fly compiler doesn't recompile the ActionScript classes if we
        // don't reference them. Put some private null references to avoid
        // weird bugs.
        private var dummyServerProgressVO : ServerProgressVO = null;
        private var dummyWorkerProgressVO : WorkerProgressVO = null;

        // Index needs function-based getter and setter, since it will do some
        // processing; and this no matter whether _index has changed or not.
        private var _index : Number = 0;

        public var nextButtonEnabled : Boolean = false;
        public var previousButtonEnabled : Boolean = false;
        public var selectedGroup : GroupVO = null;
        public var uploadedApplication : ApplicationVO = null;
        public var selectedOperation : OperationStateVO = null;

        [ArrayElementType(OperationStateVO)]
        public var operationsList : ArrayCollection = null;

        [ArrayElementType(ServerVO)]
        public var servers : ArrayCollection = null;

        [ArrayElementType(GroupVO)]
        public var groups : ArrayCollection = null;

        [ArrayElementType(ServerVO)]
        public var selectedServers : ArrayCollection = null;

        [ArrayElementType(ServerProgressVO)]
        public var serverProgressList : ArrayCollection = null;

        [ArrayElementType(WorkerProgressVO)]
        public var workerProgressList : ArrayCollection = null;

        private var groupsDelegate : GetGroupsDelegate = new GetGroupsDelegate();

        private var serversDelegate : GetServersDelegate = new GetServersDelegate();

        private var selectedGroupDelegate : GetSelectedGroupDelegate = new GetSelectedGroupDelegate();

        private var serverProgressListDelegete : GetServerProgressListDelegate = new GetServerProgressListDelegate();

        private var workerProgressListDelegete : GetWorkerProgressListDelegate = new GetWorkerProgressListDelegate();

        private var totalActiveSessionsDelegate : GetTotalActiveSessionsDelegate = new GetTotalActiveSessionsDelegate();

        private var abortServerDelegate : AbortServerDelegate = new AbortServerDelegate();

        private var restartServerDelegate : RestartServerDelegate = new RestartServerDelegate();

        private var ignoreServerDelegate : IgnoreServerDelegate = new IgnoreServerDelegate();

        private var restartWorkerDelegate : RestartWorkerDelegate = new RestartWorkerDelegate();

        private var abortWorkerDelegate : AbortWorkerDelegate = new AbortWorkerDelegate();

        private var ignoreWorkerDelegate : IgnoreWorkerDelegate = new IgnoreWorkerDelegate();

        private var createApplicationDelegate : CreateApplicationDelegate = new CreateApplicationDelegate();

        private var getUploadedApplicationDelegate : GetApplicationDelegate = new GetApplicationDelegate();

        private var checkServerDelegate : CheckServerDelegate = new CheckServerDelegate();

        private var checkWorkerDelegate : CheckWorkerDelegate = new CheckWorkerDelegate();

        private var pageIndexDelegate : GetPageIndexDelegate = new GetPageIndexDelegate();

        private var nextDelegate : NextDelegate = new NextDelegate();

        private var previousDelegate : PreviousDelegate = new PreviousDelegate();

        private var canGoToNextStepDelegate : CanGoToNextStepDelegate = new CanGoToNextStepDelegate();

        private var canGoToPreviousStepDelegate : CanGoToPreviousStepDelegate = new CanGoToPreviousStepDelegate();

        private var loadTopologyDelegate : LoadTopologyDelegate = new LoadTopologyDelegate();

        private var selectGroupDelegate : SelectGroupDelegate = new SelectGroupDelegate();

        private var selectServersDelegate : SelectServersDelegate = new SelectServersDelegate();

        private var selectOperationDelegate : SelectOperationDelegate = new SelectOperationDelegate();

        private var createNewOperationDelegate : CreateNewOperationDelegate = new CreateNewOperationDelegate();

        private var operationsListDelegate : GetOperationsListDelegate = new GetOperationsListDelegate();

        private var deleteOperationDelegate : DeleteOperationDelegate = new DeleteOperationDelegate();

        private var initializeDelegate : InitializeDelegate = new InitializeDelegate();

        public static function getInstance() : JadortModelLocator {
            if (modelLocator == null) {
                modelLocator = new JadortModelLocator();
            }
            return modelLocator;
        }

        public function JadortModelLocator() {
            super();
            if (JadortModelLocator.modelLocator != null) {
                throw new Error("Only one ModelLocator instance should be instantiated");
            }
        }

        public function getGroupsDelegate() : GetGroupsDelegate {
            return groupsDelegate;
        }

        public function getServersDelegate() : GetServersDelegate {
            return serversDelegate;
        }

        public function getSelectedGroupDelegate() : GetSelectedGroupDelegate {
            return selectedGroupDelegate;
        }

        public function getCanGoToNextStepDelegate() : CanGoToNextStepDelegate {
            return canGoToNextStepDelegate;
        }

        public function getCanGoToPreviousStepDelegate() : CanGoToPreviousStepDelegate {
            return canGoToPreviousStepDelegate;
        }

        public function getPageIndexDelegate() : GetPageIndexDelegate {
            return pageIndexDelegate;
        }

        public function getNextDelegate() : NextDelegate {
            return nextDelegate;
        }

        public function getPreviousDelegate() : PreviousDelegate {
            return previousDelegate;
        }

        public function getServerProgressListDelegete() : GetServerProgressListDelegate {
            return serverProgressListDelegete;
        }

        public function getWorkerProgressListDelegete() : GetWorkerProgressListDelegate {
            return workerProgressListDelegete;
        }

        public function getTotalActiveSessionsDelegate() : GetTotalActiveSessionsDelegate {
            return totalActiveSessionsDelegate;
        }

        public function getAbortServerDelegate() : AbortServerDelegate {
            return abortServerDelegate;
        }

        public function getRestartServerDelegate() : RestartServerDelegate {
            return restartServerDelegate;
        }

        public function getIgnoreServerDelegate() : IgnoreServerDelegate {
            return ignoreServerDelegate;
        }

        public function getAbortWorkerDelegate() : AbortWorkerDelegate {
            return abortWorkerDelegate;
        }

        public function getRestartWorkerDelegate() : RestartWorkerDelegate {
            return restartWorkerDelegate;
        }

        public function getIgnoreWorkerDelegate() : IgnoreWorkerDelegate {
            return ignoreWorkerDelegate;
        }

        public function getCheckServerDelegate() : CheckServerDelegate {
            return checkServerDelegate;
        }

        public function getCheckWorkerDelegate() : CheckWorkerDelegate {
            return checkWorkerDelegate;
        }

        public function getLoadTopologyDelegate() : LoadTopologyDelegate {
            return loadTopologyDelegate;
        }

        public function getSelectGroupDelegate() : SelectGroupDelegate {
            return selectGroupDelegate;
        }

        public function getSelectServersDelegate() : SelectServersDelegate {
            return selectServersDelegate;
        }

        public function getCreateApplicationDelegate() : CreateApplicationDelegate {
            return createApplicationDelegate;
        }

        public function getApplicationDelegate() : GetApplicationDelegate {
            return getUploadedApplicationDelegate;
        }

        public function getSelectOperationDelegate() : SelectOperationDelegate {
            return selectOperationDelegate;
        }

        public function getCreateNewOperationDelegate() : CreateNewOperationDelegate {
            return createNewOperationDelegate;
        }

        public function getOperationsListDelegate() : GetOperationsListDelegate {
            return operationsListDelegate;
        }

        public function getDeleteOperationDelegate() : DeleteOperationDelegate {
            return deleteOperationDelegate;
        }

        public function getInitializeDelegate() : InitializeDelegate {
            return this.initializeDelegate;
        }

        public function get index() : Number {
            return this._index;
        }

        // This setter is required by Index.mxml
        public function set index(index : Number) : void {
            this._index = index;
        }

        // IMPORTANT: We need an explicit setter function here, since using the
        //            ActionScript get-set design will optimize access and not
        //            call the setter if the value didn't change.
        public function setIndex(index : Number) : void {
            this.index = index;
            var getGroupsEvent : GetGroupsEvent;

            if (index == 2) {
                // Select group
                getGroupsEvent = new GetGroupsEvent();
                CairngormEventDispatcher.getInstance().dispatchEvent(getGroupsEvent);
            } else if (index == 3) {
                // Select application, refresh:
                //     - list of groups (for the list of applications)
                //     - current application
                this.uploadedApplication = null;
                getGroupsEvent = new GetGroupsEvent();
                CairngormEventDispatcher.getInstance().dispatchEvent(getGroupsEvent);
                var getApplicationEvent : GetApplicationEvent = new GetApplicationEvent();
                CairngormEventDispatcher.getInstance().dispatchEvent(getApplicationEvent);
            } else if (index == 4 || index == 5 || index == 6 || index == 8 || index == 9) {
                // Migration or maintenance progress
                CairngormEventDispatcher.getInstance().dispatchEvent(new GetServerProgressListEvent());
                CairngormEventDispatcher.getInstance().dispatchEvent(new GetWorkerProgressListEvent());
            } else if (index == 7) {
                // Select servers to maintain
                this.servers = new ArrayCollection();
                this.selectedServers = new ArrayCollection();
                CairngormEventDispatcher.getInstance().dispatchEvent(new GetServersEvent());
            }

            CairngormEventDispatcher.getInstance().dispatchEvent(new CanGoToNextStepEvent());
            CairngormEventDispatcher.getInstance().dispatchEvent(new CanGoToPreviousStepEvent());
        }
    }
}
